#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015-2016, Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import sys
from trsync.api import TRSyncApi
from trsync.cli.commands.base import TRSyncCmd


class PushCmd(TRSyncCmd):
    def get_description(self):
        return "push SRC to several DST with snapshots"

    def get_parser(self, prog_name):
        parser = super(PushCmd, self).get_parser(prog_name)
        parser.add_argument(
            'source',
            help='Source rsync url (local, rsyncd, remote shell). '
                 'Must be a directory.'
        )
        parser.add_argument(
            '-n', '--snapshot-name',
            default='',
            required=False,
            help='Snapshot name. Defaults to source url directory name. '
                 'Will contain the source/content on remote. '
                 'Full snapshot name will be "{snapshot-name}-{timestamp}". '
                 'Snapshot will be placed in directory specified by '
                 '"--snapshots-dir" option. Also "{snapshot-name}-latest" '
                 'symlink will be updated on successful sync '
                 '(--latest-successful-postfix)'
        )
        parser.add_argument(
            '-d', '--dest',
            nargs='+',
            required=True,
            help='Destination rsync url(s)'
        )
        parser.add_argument(
            '-t', '--timestamp',
            required=False,
            help='Specified timestamp will be used for snapshot. '
                 'Will be generated automaticaly by default. '
                 'Format: yyyy-mm-dd-hhMMSS'
        )
        parser.add_argument(
            '--snapshots-dir', '--snapshot-dir',
            required=False,
            default='snapshots',
            help='Directory name for snapshots relative "destination". '
                 '"snapshots" by default'
        )
        parser.add_argument(
            '--init-directory-structure',
            action='store_true',
            required=False,
            default=False,
            help='If specified, all directories including "snapshots-dir" '
                 'will be created on remote location. Disabled by default.'
        )
        parser.add_argument(
            '--snapshot-lifetime', '--save-latest-days',
            required=False,
            default=61,
            help='Snapshots for specified number of days will be saved. '
                 'All older will be removed. 61 by default. Use 0 to keep all '
                 'snapshots, "None" to keep only the latest will be deleted'
        )
        parser.add_argument(
            '--latest-successful-postfix',
            required=False,
            default='latest',
            help='Postfix for symlink to latest successfully synced snapshot. '
                 'Also used as --link-dest target. "latest" by default.'
        )
        parser.add_argument(
            '-s', '--symlinks',
            nargs='+',
            required=False,
            default=[],
            help='Update additional symlinks relative to the destination. '
                 'Only "latest" by default.'
        )
        parser.add_argument(
            '--extra',
            required=False,
            default='',
            help='String with additional rsync parameters. '
                 'E.g. it may be "\--dry-run --any-rsync-option". '
                 'Use "\\" to prevent argparse from parsing this value.'
        )

        return parser

    def take_action(self, parsed_args):
        args = vars(parsed_args)
        args['rsync_extra_params'] = self._extra(args.pop('extra'))

        results = TRSyncApi().push(args.pop('source'),
                                   args.pop('dest'),
                                   **args)
        exitcode = self._debrief(results)
        sys.exit(exitcode)
