# -*- coding: utf-8 -*-

# Copyright (c) 2015-2016, Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json
import os
import re


from trsync.objects.rsync_ops import RsyncOps


class LockError(Exception):

    def __init__(self, rsync_url, path, lockdata=None):
        self.rsync_url = rsync_url
        self.path = path
        self.lockdata = lockdata or {}

    def __str__(self):
        return "Remote path ({0}, {1}) appears to be already locked" \
               .format(self.rsync_url, self.path)


class RemoteLock(object):

    def __init__(self,
                 remote_rsync,
                 path,
                 lockdata=None,
                 lockname=".trsync.lock",
                 force=False):
        self.rsync = RsyncOps(remote_rsync.url.root)
        self.path = path
        self.force = force
        self.lockdata = lockdata
        self.lockname = lockname
        self.local_lock = os.path.join('/tmp', self.lockname)
        self.lockdata = lockdata or {}

    def _assert_not_locked(self):
        remote_locks = self.rsync.ls(
            path=self.path,
            pattern="^{0}$".format(re.escape(self.lockname))
        )
        if len(remote_locks):
            raise LockError(self.rsync.url, self.path)

    def acquire(self):
        if not self.force:
            self._assert_not_locked()

        with open(self.local_lock, "w") as lock:
            json.dump(self.lockdata, lock, indent=4)

        self.rsync.mk_dir(self.path)
        self.rsync.push(self.local_lock, self.path)
        os.remove(self.local_lock)

    def release(self):
        self.rsync.rm_file(os.path.join(self.path, self.lockname))

    def __enter__(self):
        self.acquire()

    def __exit__(self, type, value, traceback):
        self.release()
